// velosysex.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <MMSYSTEM.H>
#include <stdlib.h>


int main(int argc, char* argv[])
{

	int deviceNumber;
	unsigned int result;
	unsigned int wRtn;
	HMIDIOUT hMidiOut;
	MIDIOUTCAPS mocap;
	char pincode[4];

	deviceNumber=-1;

	if (argc != 7)
	{
		fprintf(stderr,"expecting 6 arguments: midichannel, pincode, program number, midinote, minimal pulse length, maximal pulse length\n");	
		exit(1);
	}

	int MidiChannel = atoi(argv[1]);
	if ((MidiChannel < 0) || (MidiChannel > 16))
	{
		fprintf(stderr,"MidiChannel out of range\n");	
		exit(1);
	}
	printf("MidiChannel = %i\n",MidiChannel);	
	
	if (strlen(argv[2]) != 4)
	{
		printf("pincode = %s\n",argv[2]);	
		fprintf(stderr,"Pincode must be 4 characters\n");	
		exit(1);
	}

	strncpy(pincode,argv[2],4);
	printf("pincode = %s\n",argv[2]);	
	
	int ProgramNumber = atoi(argv[3]);
	if ((ProgramNumber < 121) || (ProgramNumber > 127))
	{
		fprintf(stderr,"ProgramNumber out of range\n");	
		exit(1);
	}
	printf("ProgramNumber = %i\n",ProgramNumber);	
	
	int NoteNumber = atoi(argv[4]);
	if ((NoteNumber < 0) || (NoteNumber > 127))
	{
		fprintf(stderr,"NoteNumber out of range\n");	
		exit(1);
	}
	printf("NoteNumber = %i\n",NoteNumber);

	float MinPulse = (float)atof(argv[5]);
	if ((MinPulse < 0) || (MinPulse > 100))
	{
		fprintf(stderr,"Minimal pulse duration out of range\n");	
		exit(1);
	}
	printf("Minimal pulse duarion = %f milliseconds\n",MinPulse);


	float MaxPulse = (float)atof(argv[6]);
	if ((MaxPulse < 0) || (MaxPulse > 200))
	{
		fprintf(stderr,"Maximal pulse duration out of range\n");	
		exit(1);
	}
	printf("Maximal pulse duarion = %f milliseconds\n",MaxPulse);


	result = midiOutOpen(&hMidiOut, deviceNumber, 0, 0, CALLBACK_NULL);
	wRtn = midiOutGetDevCaps(deviceNumber, (LPMIDIOUTCAPS) &mocap, sizeof(mocap));
	if (result != MMSYSERR_NOERROR)
	{
		fprintf(stderr,"midiOutOpen: %s\n",mocap.szPname);
		char t[256];
		midiOutGetErrorText(result, t, 256);
		fprintf(stderr,"midiOutOpen: %s\n",t);
		
		exit(1);
	}
	fprintf(stderr,"midiOutOpen: Open %s as Port %d\n",mocap.szPname, deviceNumber);

	MIDIHDR MidiOutHdr;
	
	char mySysEx[266];
	mySysEx[0]=(char)0xF0;
	mySysEx[1]=(char)0x7D;
	mySysEx[2]=(char)pincode[0];
	mySysEx[3]=(char)pincode[1];
	mySysEx[4]=(char)pincode[2];
	mySysEx[5]=(char)pincode[3];

	mySysEx[6]=(char)MidiChannel & 0x7F;	// channel
	mySysEx[7]=(char)ProgramNumber & 0x7F;	// program
	mySysEx[8]=(char)NoteNumber & 0x7F;	// note

	mySysEx[9]=(char)0x00;
	mySysEx[10]=(char)0x00;
	int i;
	for(i=1;i<128;i++)
	{
		float t=MinPulse + (MaxPulse-MinPulse)*(i-1.0f)/126.0f;
		int val = (int)(t / 0.0192f);
		if (val < 0)	val=1;
		if (val > 16383)
		{ 
			val=16383;
			printf("clipping to 14 bit...");
		}
		mySysEx[9+2*i]=val/128;
		mySysEx[1+9+2*i]=val%128;
		printf("velo=%i, \tt=%.3f ms, \tc=%i cycles\n",i,t,val);
	}
	mySysEx[265]=(char)0xF7;

	MidiOutHdr.dwBufferLength=266;
	MidiOutHdr.dwBytesRecorded=266;
	MidiOutHdr.lpData=mySysEx;
	MidiOutHdr.dwFlags=0;

	result = midiOutPrepareHeader(hMidiOut, &MidiOutHdr, sizeof(MIDIHDR));

	if (result != MMSYSERR_NOERROR)
	{
		char t[256];
		midiOutGetErrorText(result, t, 256);
		fprintf(stderr,"midiOutPrepareHeader: %s\n",t);
		
		exit(1);
	}

	
	result = midiOutLongMsg(hMidiOut, &MidiOutHdr, sizeof(MIDIHDR));

	if (result != MMSYSERR_NOERROR)
	{
		char t[256];
		midiOutGetErrorText(result, t, 256);
		fprintf(stderr,"midiOutLongMsg: %s\n",t);
		
		exit(1);
	}

	printf("closing mid port...\n");
	midiOutReset(hMidiOut);
	midiOutClose(hMidiOut);

	return 0;
}

